-- construct parent table

create table `ombutel`.`ombu_trunk_parameters` (
	`trunk_parameter_id` int unsigned not null primary key auto_increment,
	`trunk_id` int unsigned not null,
	`enabled` enum('no','yes') not null default 'yes',
	`sort` int unsigned not null default 0,
	foreign key (`trunk_id`) references `ombutel`.`ombu_trunks` (`trunk_id`) on delete cascade
);

create temporary table `ombutel`.`params` (
	`rowid` int unsigned not null,
	`trunk_parameter_id` int unsigned not null default 0,
	`trunk_id` int unsigned not null default 0,
	`type` varchar(255) not null default '',
	`param` varchar(255) not null default '',
	`value` varchar(255) not null default '',
	`enabled` enum('no','yes') not null default 'yes',
	`sort` int unsigned not null default 0
) collate=utf8_unicode_ci;

set @rowid = 0;

insert into `ombutel`.`params` (`rowid`, `trunk_id`, `type`, `param`, `value`, `enabled`, `sort`)
select (@rowid := @rowid + 1), `trunk_id`, `type`, `param`, `value`, if(`enabled`='no','yes','no'), `sort`
from `ombutel`.`ombu_sip_trunk_parameters`;

insert into `ombutel`.`params` (`rowid`, `trunk_id`, `type`, `param`, `value`, `enabled`, `sort`)
select (@rowid := @rowid + 1), `trunk_id`, `type`, `param`, `value`, if(`enabled`='no','yes','no'), `sort`
from `ombutel`.`ombu_iax_trunk_parameters`;

insert into `ombutel`.`params` (`rowid`, `trunk_id`, `type`, `param`, `value`, `enabled`, `sort`)
select (@rowid := @rowid + 1), `trunk_id`, `type`, `param`, `value`, if(`enabled`='no','yes','no'), `sort`
from `ombutel`.`ombu_h323_trunk_parameters`;

insert into `ombutel`.`params` (`rowid`, `trunk_id`, `type`, `param`, `value`, `enabled`, `sort`)
select (@rowid := @rowid + 1), `trunk_id`, '', `param`, `value`, if(`enabled`='no','yes','no'), `sort`
from `ombutel`.`ombu_telephony_trunk_parameters`;

insert into `ombutel`.`ombu_trunk_parameters` (`trunk_id`, `enabled`, `sort`)
select `trunk_id`, `enabled`, `sort`
from `ombutel`.`params`
order by `rowid`;

set @parameter_id = greatest(last_insert_id(), 1) - 1;

update `ombutel`.`params` set
	`trunk_parameter_id` = (@parameter_id := @parameter_id + 1)
order by `rowid`;

-- rebuild sip parameters

drop table `ombutel`.`ombu_sip_trunk_parameters`;

create table `ombutel`.`ombu_sip_trunk_parameters` (
	`trunk_parameter_id` int unsigned not null,
	`type` varchar(255) not null default 'friend',
	`param` varchar(255) not null default '',
	`value` varchar(255),
	foreign key (`trunk_parameter_id`) references `ombutel`.`ombu_trunk_parameters` (`trunk_parameter_id`) on delete cascade
) collate=utf8_unicode_ci;

insert into `ombutel`.`ombu_sip_trunk_parameters` (`trunk_parameter_id`, `type`, `param`, `value`)
select p.`trunk_parameter_id`, p.`type`, p.`param`, p.`value`
from `ombutel`.`params` p
left join `ombutel`.`ombu_trunks` t using (`trunk_id`)
where t.`technology` = 'sip'
order by p.`rowid`;

-- rebuild iax parameters

drop table `ombutel`.`ombu_iax_trunk_parameters`;

create table `ombutel`.`ombu_iax_trunk_parameters` (
	`trunk_parameter_id` int unsigned not null,
	`type` varchar(255) not null default 'friend',
	`param` varchar(255) not null default '',
	`value` varchar(255),
	foreign key (`trunk_parameter_id`) references `ombutel`.`ombu_trunk_parameters` (`trunk_parameter_id`) on delete cascade
) collate=utf8_unicode_ci;

insert into `ombutel`.`ombu_iax_trunk_parameters` (`trunk_parameter_id`, `type`, `param`, `value`)
select p.`trunk_parameter_id`, p.`type`, p.`param`, p.`value`
from `ombutel`.`params` p
left join `ombutel`.`ombu_trunks` t using (`trunk_id`)
where t.`technology` = 'iax'
order by p.`rowid`;

-- rebuild h323 parameters

drop table `ombutel`.`ombu_h323_trunk_parameters`;

create table `ombutel`.`ombu_h323_trunk_parameters` (
	`trunk_parameter_id` int unsigned not null,
	`param` varchar(255) not null default '',
	`value` varchar(255),
	foreign key (`trunk_parameter_id`) references `ombutel`.`ombu_trunk_parameters` (`trunk_parameter_id`) on delete cascade
) collate=utf8_unicode_ci;

insert into `ombutel`.`ombu_h323_trunk_parameters` (`trunk_parameter_id`, `param`, `value`)
select p.`trunk_parameter_id`, p.`param`, p.`value`
from `ombutel`.`params` p
left join `ombutel`.`ombu_trunks` t using (`trunk_id`)
where t.`technology` = 'h323'
order by p.`rowid`;

-- rebuild telephony parameters

drop table `ombutel`.`ombu_telephony_trunk_parameters`;

create table `ombutel`.`ombu_telephony_trunk_parameters` (
	`trunk_parameter_id` int unsigned not null,
	`param` varchar(255) not null default '',
	`value` varchar(255),
	foreign key (`trunk_parameter_id`) references `ombutel`.`ombu_trunk_parameters` (`trunk_parameter_id`) on delete cascade
) collate=utf8_unicode_ci;

insert into `ombutel`.`ombu_telephony_trunk_parameters` (`trunk_parameter_id`, `param`, `value`)
select p.`trunk_parameter_id`, p.`param`, p.`value`
from `ombutel`.`params` p
left join `ombutel`.`ombu_trunks` t using (`trunk_id`)
where t.`technology` = 'telephony'
order by p.`rowid`;
