function xepm_mapping() {
	this.url = "./";
	this.models = [];
	this.templates = [];
}

xepm_mapping.prototype.scan_devices = function() {
	var filter_results = !!$("input#filter_scan").prop("checked");
	var subnet_field = $("input#xepm_subnet");
	var subnet = subnet_field.val();
	if(subnet.length !== 0) {
		var scan_button = $("td.scan_subnet_button");
		scan_button.attr('disabled','disabled');

		xepm_mapping.clear_unassinged_devices();
		$('table.unassigned_table').css({'display':'none'}); //Hide unassigned devices table
		$('i#scan_subnet_spinner').css({'display':'block'}); //Show spinner

		$.ajax({
			type: "POST",
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'scan_subnet',
				scan_subnet: subnet
			},
			success: function (data) {
				xepm_mapping.scanned_devices = data;
				var table = $('div.epm_unassigned_devices > table').find('tbody');
				$.each(data, function(index, device) {
					if (!filter_results || (this.brand_id > 0)) {
						var brands = $('select#epm_device_mapping_brands').clone(); // Clone brands dropdown field
						brands.attr({
							'id': null,
							'name' : 'unmapped_brand_id[]'
						});
						xepm_mapping.add_device(this.display_mac_address, this.ip_address, this.brand_id);
					}
				});
				$('table.unassigned_table').css({'display':'block'}); //Unhide unassigned devices table
				scan_button.removeAttr('disabled');
			},
			complete: function (data) {
				$('i#scan_subnet_spinner').css({'display':'none'});
			}
		});
	}
};

// Remove all unassigned devices
xepm_mapping.prototype.clear_unassinged_devices = function() {
	$('div.epm_unassigned_devices > table > tbody').find('tr').remove();
};

// Opens new browsers tab with device cfg.
xepm_mapping.prototype.view_cfg = function(device_id) {
	var url = `/?class=epm_device_mapping&method=view_config&data=${device_id}`;
	window.open(url, '_blank');
};

// Move assigned device to unassigned on delete device click
xepm_mapping.prototype.remove_device = function(element) {
	var row = $(element).parent().parent();
	var mac_address = row.children().eq(1).text();
	row.remove();
	xepm_mapping.add_device(mac_address, '');
};

// Create a new unmapped device
xepm_mapping.prototype.add_device = function(mac_address, ip_address, brand_id) {
	var models = $("<select/>", { name : "unmapped_model_id[]" }).append($("<option/>", { selected : true }));
	var templates = $("<select/>", { name : "unmapped_template_id[]" }).append($("<option/>", { selected : true }));
	var brands = $('select#epm_device_mapping_brands').clone(); // Clone brands dropdown field
	brands.attr({
		'id': null,
		'name' : 'unmapped_brand_id[]'
	});

	var row = $("<tr/>")
		.append($("<td/>").append($("<input/>", { type : "text", value : mac_address, name: "unmapped_mac_address[]" })))
		.append($("<td/>", { html: (ip_address === null) ? "-" : ip_address }))
		.append($("<td/>").append(brands))
		.append($("<td/>").append(models))
		.append($("<td/>").append(templates))
		.append($("<td/>")); // extensions

	if (brand_id > 0) {
		brands.val(brand_id);
		xepm_mapping.populate_on_brand_selection(row, brand_id);
	}

	brands.bind("change keyup", function(event) { xepm_mapping.handle_dropdown(event); });
	models.bind("change keyup", function(event) { xepm_mapping.handle_dropdown(event); });

	row.appendTo($("div.epm_unassigned_devices > table").find('tbody'));
	$('table.unassigned_table').css({'display':'block'}); //Unhide unassigned devices table
};

// Populate model dropdown on brand selection
xepm_mapping.prototype.populate_on_brand_selection = function(row, brand_id) {
	if (xepm_mapping.models[brand_id] === undefined) {
		var element = row.find("select[name*='model_id']");
		$.ajax({
			type: 'POST',
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'models_by_brand',
				brand_id: brand_id
			},
			success: function (data) {
				element.fill(data.models, function(model) {
					this.val(model.model_id);
					this.text(model.name);
				});
				xepm_mapping.populate_templates(element, row, data);
				element.data("options_loaded", true);
			}
		});
	}
};

xepm_mapping.prototype.reindex_mapped_extensions = function() {
	$("table.assigned_devices_table select[name*='mapped_extensions']").each(function(index, element) {
		element = $(element);
		var index = element.closest("tr").data("index");
		element.attr("name", element.attr("name")
			.replace("mapped_extensions[0]", "mapped_extensions["+index+"]"));
	});
};

xepm_mapping.prototype.reindex_unmapped_extensions = function() {
	$("table.unassigned_table select[name*='unmapped_extensions']").each(function(index, element) {
		element = $(element);
		var index = element.closest("tr")[0].rowIndex - 1;
		element.attr("name", element.attr("name")
			.replace("unmapped_extensions[0]", "unmapped_extensions["+index+"]"));
	});
};

xepm_mapping.prototype.populate_templates = function(element, row, data) {
	var prefix = row.closest("table")[0].id.contains("unassigned") ? "unmapped" : "mapped";
	row.find("select[name*='template_id']").fill(data.templates, function(template) {
		this.val(template.template_id);
		this.text(template.name);
	});
	var cell = row.children().eq(5);
	row.find("select[name*='extensions']").remove();
	for (var i = 0; i < data.extension_count; i++) {
		var dropdown = $('<select/>', { name: prefix + "_extensions[0][" + i + "]"});
		dropdown.fill(data.extensions, function(extension) {
			this.val(extension.value);
			this.text(extension.content);
		}, ['', '']);
		dropdown.appendTo(cell);
	}
	row.find("select[name*='extensions']").data("options_loaded", true);
	if (prefix == "mapped") {
		xepm_mapping.reindex_mapped_extensions();
	} else {
		xepm_mapping.reindex_unmapped_extensions();
	}
}

// Polulate template and extension dropdowns on model selection
xepm_mapping.prototype.populate_on_model_selection = function(row, model_id) {
	if (xepm_mapping.templates[model_id] === undefined) {
		var element = row.find("select[name*='extensions']");
		$.ajax({
			type: 'POST',
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'templates_and_ext_by_model',
				model_id: model_id
			},
			success: function(data) { xepm_mapping.populate_templates(element, row, data); }
		});
	}
};

//NOTE:
// If there any change, warn.
xepm_mapping.prototype.configure_button_listener = function(device_id) {
	if(!xepm_mapping.is_changed) {
		//Do something
	} else if(confirm("Are you sure you want to nevigate away? All unsaved changes will be lost")) {
		//Do something else
	}
};

// assign events to page elements
xepm_mapping.prototype.assign_events = function() {
	$("select").bind("change keyup", function(event) {
		xepm_mapping.is_changed = true;
		xepm_mapping.handle_dropdown(event);
	});
	$("input").bind("change keyup", function(event) {
		if (event.target.id != "xepm_subnet") {
			xepm_mapping.is_changed = true;
		}
	});
	$("input[name*='filter_scan']").bind("change", function(event) {
		var filter_results = !!$(event.target).prop("checked");
		if (xepm_mapping.scanned_devices !== undefined) {
			$("div.epm_unassigned_devices > table").find("tbody").empty();
			$.each(xepm_mapping.scanned_devices, function() {
				if (!filter_results || (this.brand_id > 0)) {
					xepm_mapping.add_device(this.display_mac_address, this.ip_address, this.brand_id);
				}
			});
		}
	});
};

// Process dropdown events
xepm_mapping.prototype.handle_dropdown = function(event) {
	var element = event.target;
	if (element.previous_value != element.value) {
		element.previous_value = element.value;
		var row = $(element).closest("tr");
		if (element.name.contains("brand_id")) {
			xepm_mapping.populate_on_brand_selection(row, element.value);
		} else if (element.name.contains("model_id")) {
			xepm_mapping.populate_on_model_selection(row, element.value);
		}
	}
};

// Reboot selected devices
xepm_mapping.prototype.reboot_devices = function() {
	var devices = [];
	$("input[type='checkbox']:checked").each(function() {
		devices[devices.length] = $(this).next().val();
		$(this).prop("checked", false);
	});
	if (devices.length > 0) {
		$.ajax({
			type: "POST",
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'reset_devices',
				reset_devices : devices
			}
		});
	}
};

var found_elements = [];
var search_value = null;
function search_extension() {
	var found = false;
	var search = $('#epm_device_mappinggeneral div.search_field input#extension_search');
	if(search.val().length > 0) {
		var pattern = new RegExp(search.val().toLowerCase());
		var selects = $('#epm_device_mappinggeneral table.assigned_devices_table select.mapped_extensions option:selected');
		if(!(found_elements.length != 0 && search_value == search.val())) {
			found_elements = [];
			search_value = search.val();
			selects.each(function(index, value) {
				var elem = $(this)[0].parentElement;
				$(elem).removeClass('highlight');
				if(($(this).text()).length > 0) {
					if(pattern.exec($(this).text())) {
						found = true;
						found_elements.push($(elem));
						$(elem).addClass('highlight');
						$(elem).focus();
					}
				}
			});
		} else {
			found = true;
		}
		if(!found) {
			BootstrapDialog.show({
				type: 'device_mapping_search',
				title: device_mapping_title,
				message: device_mapping_no_extension,
				buttons: [{
					label: device_mapping_confirm,
					cssClass: 'btn-primary',
					action: function(dialogItself) {
						dialogItself.close();
					}
				}]
			});
		} else {
			focus_extension();
		}
	}
}

var search_idx = 0;
function focus_extension() {
	var elem = found_elements[search_idx];
	if(found_elements.length-1 > search_idx) {
		search_idx++;
	} else {
		search_idx = 0;
	}
	$(elem).focus();
	var center = $(document).height()/2;
	var dummy_elem = $('<div></div>').css({ //Use dummy element
		position: 'absolute', top: $(elem).position().top - center
	}).appendTo($(elem).parent());
	dummy_elem[0].scrollIntoView();
	dummy_elem.remove();
}

var xepm_mapping = new xepm_mapping();
xepm_mapping.assign_events();

function ip2long(value) {
	var ip = value.match(/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/);
	if (ip) {
		return (+ip[1] << 24) + (+ip[2] << 16) + (+ip[3] << 8) + (+ip[4]);
	}
	return false;
}

function set_default_subnet() {
	if (ip = ip2long(window.location.host)) {
		for (i = 0; i < subnets.length; i++) {
			tmp = subnets[i].split('/');
			network = ip2long(tmp[0]);
			mask = -1 << (32 - tmp[1]);
			if ((ip & mask) == network) {
				$("#epm_device_mappinggeneral #xepm_subnet").val(subnets[i]);
				return;
			}
		}
	}
	if (subnets.length > 0) {
		$("#epm_device_mappinggeneral #xepm_subnet").val(subnets[0]);
	}
}

$(function() {
	set_default_subnet();
	$("#epm_device_mappinggeneral").on("focus", "[name^='mapped'],[name^='unmapped_extensions']", function() {
		var element = $(this);
		var current_value = element.val();
		var device_id = $(this).closest('tr').find('input[id="mapped_device_id[]"]').val();
		if (!element.data("options_loaded")) {
			element.prop("disabled", true);
			element.addClass("loading");
			switch(element[0].getAttribute('name')) {
				case 'mapped_template_id[]': {
					method = 'template_by_device'; // template
					break;
				}
				case 'mapped_model_id[]': {
					method = 'model_by_device'; // model
					break;
				}
				case 'mapped_brand_id[]': {
					method = 'brands'; // brand
					break;
				}
				default: {
					method = 'extensions';
				}
			}
			$.ajax({
				type: "POST",
				url: xepm_mapping.url,
				dataType: 'json',
				data: {
					class: 'endpoint',
					method: method,
					device_id: device_id
				},
				success: function (data) {
					element.empty();
					element.append(new Option('', ''));
					if(data.hasOwnProperty('templates')) {
						$.each(data.templates, function(index, template) {
							element.append(new Option(template.name, template.template_id));
							if(current_value == template.template_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					} else if(data.hasOwnProperty('extensions')) {
						$.each(data.extensions, function(index, extension) {
							element.append(new Option(extension.name, extension.template_id));
							if(current_value == extension.template_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					} else if(data.hasOwnProperty('models')) {
						$.each(data.models, function(index, model) {
							element.append(new Option(model.name, model.model_id));
							if(current_value == model.model_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					} else if(data.hasOwnProperty('brands')) {
						$.each(data.brands, function(index, brand) {
							element.append(new Option(brand.name, brand.brand_id));
							if(current_value == brand.brand_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					}
				}
			}).done(function (data) {
				element.data("options_loaded", true);
				element.removeClass("loading");
				element.prop("disabled", false);
			});
		}
		return false;
	});

	$('#epm_device_mappinggeneral span.extension_search_icon').on('click', search_extension);

	$('form#epm_device_mapping_form').keydown(function(event) { //prevent form submition on "enter"
		if(event.keyCode == 13) {
			event.preventDefault();
			search_extension();
			return false;
		}
	});

	$('#epm_device_mappinggeneral button.check_uncheck').on('click', function() {
		var checkboxs = $('#epm_device_mappinggeneral input#checkbox_mapped_brand_id\\[\\]');
		if($(this).hasClass('all_checked')) {
			checkboxs.prop('checked', false);
		} else {
			checkboxs.prop('checked', true);
		}
		$(this).toggleClass('all_checked');
	});

	$("#epm_device_mappinggeneral").on("update_mac", function(event, args) {
		$(`#epm_device_mappinggeneral .assigned_devices_table tr[data-index="${args.id}"] td:nth-of-type(2)`).text(args.mac);
	});
});
